## $Id: browser.py 12478 2015-01-15 22:12:25Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from waeup.kofa.interfaces import CLEARED
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from kofacustom.wdu.students.interfaces import ICustomStudentOnlinePayment
from kofacustom.wdu.applicants.interfaces import ICustomApplicantOnlinePayment
from kofacustom.wdu.interfaces import MessageFactory as _

PRODUCT_ID = '6207'
SITE_NAME = 'wdu.waeup.org'
PROVIDER_ACCT = '0137712635'
PROVIDER_BANK_ID = '10'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'WDU'
CURRENCY = '566'
GATEWAY_AMT = 300.0

POST_ACTION = 'https://stageserv.interswitchng.com/test_paydirect/pay'
#POST_ACTION = 'https://webpay.interswitchng.com/paydirect/pay'

#HOST = 'webpay.interswitchng.com'
HOST = 'stageserv.interswitchng.com'
HTTPS = True

#URL = '/paydirect/services/TransactionQueryWs.asmx'
URL = '/test_paydirect/services/TransactionQueryWS.asmx'

httplib.HTTPSConnection.debuglevel = 0

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = 'CEF793CBBE838AA0CBB29B74D571113B4EA6586D3BA77E7CFA0B95E278364EFC4526ED7BD255A366CDDE11F1F607F0F844B09D93B16F7CFE87563B2272007AB3'

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_acct_1'] = '4742014159'
        xmldict['institution_bank_id_1'] = '47'
        xmldict['institution_amt_1'] = '0.0'
        xmldict['institution_acct_2'] = '0029688237'
        xmldict['institution_bank_id_2'] = '11'
        xmldict['institution_amt_2'] = '0.0'
        provider_amt = 4000.0
        self.pay_item_id = '101'
        # other charges + tech fee
        if self.context.student.state == CLEARED:
            institution_amt_2 = 207500.0 + 1000.0
        else :
            institution_amt_2 = 150500.0 + 1000.0
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt_1'] = 100 * (
            self.context.amount_auth - provider_amt - GATEWAY_AMT - institution_amt_2)
        xmldict['institution_amt_2'] = 100 * institution_amt_2

        # Interswitch amount is not part of the xml data
        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt_1)d" bank_id="%(institution_bank_id_1)s" acct_num="%(institution_acct_1)s" />
<item_detail item_id="2" item_name="other charges and tech fee" item_amt="%(institution_amt_2)d" bank_id="%(institution_bank_id_2)s" acct_num="%(institution_acct_2)s" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT

        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return


class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    https = HTTPS

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    https = HTTPS
